
#include "system.h"
#include "board.h"
#include "checker.h"
#include "enums.h"
#include "move.h"

#include <iostream>
#include <fstream>
using namespace std;

Board::Board() {
	char totalCheckers = 'a';
	//int owner = 1;
	System& system = System::getInstance();
	const Player* owner = system.getPlayer(MIN_PLAYER);

	for (size_t curY = 0;curY < BOARD_HEIGHT;++curY) {

		if (curY == 4) {
			totalCheckers = 'A';		//Make uppercase
			owner = system.getPlayer(MAX_PLAYER);			//Swap owners
		}

		for (size_t curX = 0;curX < BOARD_WIDTH;++curX) {
			pieces[curX][curY] = NULL;
			if ((curY % 2 == 0 && curX % 2 != 0) ||
				(curY % 2 != 0 && curX % 2 == 0))
			{
				if (curY < 3) {
					pieces[curX][curY] = new Checker(totalCheckers, curX, curY, owner);
					totalCheckers++;
				} else if (curY > 4) {
					pieces[curX][curY] = new Checker(totalCheckers, curX, curY, owner);
					totalCheckers++;
				}
			}

		}
	}
	simulation = false;
}

Board::Board(Board* board, Checker* checker) {
	simulation = true;
	for (size_t curY = 0;curY < BOARD_HEIGHT;++curY) {
		for (size_t curX = 0;curX < BOARD_WIDTH;++curX) {
			if (checker && board->pieces[curX][curY] == checker)
				pieces[curX][curY] = new Checker(checker, this);	//Create the checker we're simulating
			else
				pieces[curX][curY] = board->pieces[curX][curY];
		}
	}
}

//Delete any simulated checkers & any normal checkers
Board::~Board() {
	for (size_t curY = 0; curY < BOARD_HEIGHT;++curY) {
		for (size_t curX = 0; curX < BOARD_WIDTH; ++curX) {
			DeleteChecker(pieces[curX][curY]);
			pieces[curX][curY] = NULL;
		}
	}
}

long int Board::Rate(const Player* justMoved, MoveCode moveCode, /*Checker* moved,*/ Direction dir) {

	const int W_BADSELECT = -3;
	const int W_DIST = 4;

	long int heuristic = 0;	//The return value
	int mult = (justMoved->getType() == MIN_PLAYER) ? -1 : 1;
	//Direction dir;
	Checker* c;
	MoveCode mc;
	//int i;
	Move move;
	int mincheckersMissing = 12; 	//# of min's checkers missing
	int maxcheckersMissing = 12;
	int winRow;
	int moveRating;

	//See how good our move was
	heuristic = move.Rate(moveCode, mult);

	if (heuristic == MIN_WIN || heuristic == MAX_WIN)
		return heuristic;

	//And examine the board:
	for (int curY = 0; curY < BOARD_HEIGHT; ++curY) {
		for (int curX = 0; curX < BOARD_WIDTH; ++curX) {
			c = pieces[curX][curY];
			if (!c)
				continue;

			//How many checkers are left?
			if (c->getOwner()->getType() == MIN_PLAYER) {
				mult = -1;
				mincheckersMissing--;
				dir = DIR_DN_LEFT;
			} else {
				maxcheckersMissing--;
				mult = 1;
				dir = DIR_UP_LEFT;
			}

			//Enemy response? (Factors into 'how good was my move')
			for (int i = 0; i < 2; ++i) {
				move.Init(c->getX(), c->getY(), dir, c, this);
				mc = move.CalcType();
				if (c->getOwner() == justMoved)
					moveRating = move.Rate(mc, mult, true);
				else
					moveRating = move.Rate(mc, mult, false);

				if (moveRating == MIN_WIN || moveRating == MAX_WIN)
					return moveRating;
				heuristic += moveRating;
				dir = SwapLeftRight(dir);
			}

			winRow = (c->getOwner()->getType() == MIN_PLAYER) ? MIN_WIN_Y : MAX_WIN_Y;
			//Dist from checker to goal, but sub from max dist (smaller distance is better)
			heuristic += mult * ((int)BOARD_HEIGHT - 1 - abs(winRow - c->getY())) * W_DIST;
		}
	}

	//x pts for each captured checker prettymuch
	//this effectively 'carries' a bit of the weight btwn states

	//NOTE: Uncommenting this plays to a stalemate on top depth 5 and bottom depth 3
	heuristic += -1 * W_BADSELECT * mincheckersMissing;
	heuristic += 1 * W_BADSELECT * maxcheckersMissing;

	return heuristic;
}

Checker* Board::getChecker(char index) const {
	for (size_t curY = 0; curY < BOARD_HEIGHT; ++curY) {
		for (size_t curX = 0; curX < BOARD_WIDTH; ++curX) {
			if (pieces[curX][curY] && pieces[curX][curY]->getIndex() == index)
				return pieces[curX][curY];
		}
	}
	return NULL;
}

Direction SwapLeftRight(Direction dir) {
	if (dir == DIR_UP_LEFT)
		return DIR_UP_RIGHT;
	else if (dir == DIR_UP_RIGHT)
		return DIR_UP_LEFT;
	else if (dir == DIR_DN_LEFT)
		return DIR_DN_RIGHT;
	else //dIR_DN_RIGHT
		return DIR_DN_LEFT;
}

void Board::DeleteChecker(Checker* checker) {
	if (!checker)
		return;
	//Delete checkers that belong to this simulation [don't delete other boards' simulated checkers]
	//If it's not a simulation, it's the current board, and all checkers belong to it
	pieces[checker->getX()][checker->getY()] = NULL;
	if (!simulation || (simulation && checker->isSimulation() && checker->getBoard() == this)) {
		delete checker;
		checker = NULL;
	}
 }

void Board::Draw(bool file) {

	ostream* pout;
	ofstream filestream;

	if (file) {
		filestream.open("rating.txt", ios::app);
		pout = &filestream;
	} else {
		pout = &cout;
	}

	//whatever
	ostream& out = *pout;

	//One whitespace on each side, # of borders = # of squares + 1
	//+---+---+---+---+---+---+---+---+
	//|   |   |   |   |   |   |   |   |
	//+---+---+---+---+---+---+---+---+
	//|   |   |   |   |   |   |   |   |
	//+---+---+---+---+---+---+---+---+
	//...

	//totalWidth/totalHeight:
	//# of squares (BOARD_WIDTH) expanded out (*) to include 1 border per square (+1)
	//Whitespace on each side of checker per square (2*white), one piece per square (+1),
	//and one more total border (total+1)

	size_t totalWidth = BOARD_WIDTH*(2*HORIZ_WHITESPACE+2)+1;
	size_t totalHeight = BOARD_HEIGHT*(2*VERT_WHITESPACE+2)+1;
	int checkerX = 0;
	int checkerY = 0;

	for (size_t curY = 0;curY < totalHeight;++curY) {
		for (size_t curX = 0;curX < totalWidth;++curX) {
			if (curY % 2 != 0) {		//Row is odd
				if (curX % 4 == 2) {	//Every 4th one starting at 2
					if (pieces[checkerX][checkerY] != NULL)
						pieces[checkerX][checkerY]->Draw(out);
					else
						out << " ";
					checkerX++;
					if (checkerX >= BOARD_WIDTH)
						checkerX = 0;
				} else if (curX % 4 == 0){
					out << "|";
				} else {
					out << " ";
				}
			} else if (curX % 4 == 0) {
				out << "+";
			} else {
				out << "-";
			}
		}
		out << endl;
		if (curY % 2 != 0)
			checkerY++;
	}
	if (file)
		filestream.close();
}