
#ifndef BOARD_H
#define BOARD_H

#include "enums.h"
#include "player.h"
#include <vector>
using namespace std;

extern enum Direction;
extern enum MoveCode;
class Checker;
class Move;
class Renderer;

Direction SwapLeftRight(Direction dir);

class Board {
private:
	friend class Move;
	friend class Renderer;

	//CONSTs relating to board
	static const size_t BOARD_WIDTH = 8;
	static const size_t BOARD_HEIGHT = 8;
	static const size_t MIN_WIN_Y = BOARD_HEIGHT-1;
	static const size_t MAX_WIN_Y = 0;

	//CONSTs relating to board rendering
	static const size_t HORIZ_WHITESPACE = 1;	//Whitespace on each side of a checker
	static const size_t VERT_WHITESPACE = 0;

	static const char BORDER_HORIZ = '-';
	static const char BORDER_VERT = '|';
	static const char BORDER_INTERSECT = '+';

	//Correctly handle deletion
	void DeleteChecker(Checker* checker);

	Checker* pieces[BOARD_WIDTH][BOARD_HEIGHT];
	bool simulation;

public:

	//Is it valid to move "dir" from x, y?
	//MoveCode isValidMove(int toX, int toY, Direction heading, int onCheckerLast = 0, bool multicheck = false);
	bool isSimulation() { return simulation; }

	void Draw(bool file = false);
	Checker* getChecker(char index) const;
	Checker* getChecker(size_t x, size_t y) { return (x < BOARD_WIDTH && y < BOARD_HEIGHT) ? pieces[x][y] : NULL; }
	Move MoveChecker(Checker* checker, Direction dir, bool simulate = false);

	long int Rate(const Player* forPlayer, MoveCode moveCode, /*Checker* moved,*/ Direction dir);

	Board();
	Board(Board* board, Checker* checker);	//Init simulation
	~Board();
};

#endif