
#include "renderer.h"
#include "system.h"

#include <sdl.h>
#include <sdl_image.h>
#include <sdl_ttf.h>
#include <iostream>

using namespace std;

inline Sint16 Renderer::ToScreenCoord(int num) {
	return Sint16(num * SPACE_GEN + CHECKER_OFFSET);
}

inline Sint16 Renderer::Snap(Sint16 num) {
	return Sint16(num/SPACE_GEN * SPACE_GEN);
}

bool Renderer::Inside(int testx, int testy, int boundxlo, int w, int boundylo, int h) {
	return (testx >= boundxlo && testx <= boundxlo+w && testy >= boundylo && testy <= boundylo+h) ? true : false;
}

void Renderer::Draw() {

	System& sys = System::getInstance();
	const Board& board = sys.getBoard();

	SDL_BlitSurface(textures[BOARD], NULL, display, &other[BOARD]);
	SDL_BlitSurface(textures[SIDEBAR], NULL, display, &other[SIDEBAR]);

	UI uikey = REDTURN;
	if (sys.win == 1)
		uikey = REDWINS;
	else if (sys.win == 2)
		uikey = BLACKWINS;
	else if (sys.getTurn()->getType() == MAX_PLAYER)
		uikey = BLACKTURN;

	SDL_BlitSurface(textures[(char)uikey], NULL, display, &ui[uikey]);

	if (sys.getTurn()->isThinking())
		SDL_BlitSurface(textures[THINKING], NULL, display, &ui[THINKING]);

	Keys key = BADBOX;
	Sint16 checkx;
	Sint16 checky;

	if (onBoard) {

		checkx = other[BOX].x + CHECKER_OFFSET;
		checky = other[BOX].y + CHECKER_OFFSET;
		other[BOX].w = SPACE_W;
		other[BOX].h = SPACE_H;

		if (sys.getTurn()->isComputer() == false) {
			for (map<char, SDL_Rect>::iterator i = checkers.begin(); i != checkers.end(); ++i) {
				if (checkx == i->second.x && checky == i->second.y) {
					Checker* curChecker = board.getChecker(i->first);
					if (curChecker && curChecker->getOwner() == sys.getTurn())
						key = BOX;
					break;
				}
			}
		}
	} else {

		checkx = other[BOX].x;
		checky = other[BOX].y;
		for (map<UI, SDL_Rect>::iterator i = buttons.begin(); i != buttons.end(); ++i) {
			if (Inside(checkx, checky, i->second.x, i->second.w, i->second.y, i->second.h)) {
				other[BOX].x = i->second.x;
				other[BOX].y = i->second.y;
				other[BOX].w = i->second.w;
				other[BOX].h = i->second.h;
				key = BOX;
				break;
			}
		}
	}

	SDL_Rect* src = NULL;
	if (textures[(char)key]->w != SPACE_GEN)
		src = &other[key];

	if (other[key].x != -1)
		SDL_BlitSurface(textures[(char)key], src, display, &other[key]);

	if (other[SELECTION].x != -1)
		SDL_BlitSurface(textures[SELECTION], NULL, display, &other[SELECTION]);

	for (map<UI, SDL_Rect>::iterator i = buttons.begin(); i != buttons.end(); ++i)
		SDL_BlitSurface(textures[char(i->first)], NULL, display, &(i->second));

	char color = RED;
	for (char i = 'a'; i != 'M'; ++i) {

		if (i == 'm') {
			i = ('A'-1);
			color = BLACK;
			continue;
		}

		if (checkers[i].x == -1)
			continue;
		SDL_BlitSurface(textures[color], NULL, display, &checkers[i]);
	}
	SDL_UpdateRect(display, 0, 0, 0, 0);
}

bool Renderer::Init() {

	const SDL_Color redcolor	= {150, 0, 0, 0};
	const SDL_Color blackcolor	= {0, 0, 0, 0};
	const SDL_Color bluecolor	= {0, 0, 150, 0};

	if (SDL_Init(SDL_INIT_VIDEO) == -1) {
		cerr << "Error on init." << endl;
		cin.get();
		return false;
	}
	atexit(SDL_Quit);

	display = SDL_SetVideoMode(SCREEN_W, SCREEN_H, 0, SDL_ANYFORMAT);

	TTF_Init();
	atexit(TTF_Quit);

	if (!display)
		return false;

	textures[BLACK]		= IMG_Load("images/black.png");
	textures[BLACKSIDE]	= IMG_Load("images/blackside.png");
	textures[RED]		= IMG_Load("images/red.png");
	textures[REDSIDE]	= IMG_Load("images/redside.png");
	textures[BOARD]		= IMG_Load("images/board.png");
	textures[SELECTION]	= SDL_CreateRGBSurface(SDL_HWSURFACE, SPACE_W, SPACE_H, 32, 0, 0, 0, 0);
	textures[BOX]		= SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_W, SCREEN_H, 32, 0, 0, 0, 0);
	textures[BADBOX]	= SDL_CreateRGBSurface(SDL_HWSURFACE, SPACE_W, SPACE_H, 32, 0, 0, 0, 0);
	textures[SIDEBAR]	= SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_W-textures[BOARD]->w, SCREEN_H, 32, 0, 0, 0, 0);

	TTF_Font* font		= TTF_OpenFont("cour.ttf", 20);
	textures[REDTURN]	= TTF_RenderText_Solid(font, "Red's turn", redcolor);
	textures[BLACKTURN]	= TTF_RenderText_Solid(font, "Black's turn", blackcolor);
	textures[THINKING]	= TTF_RenderText_Solid(font, "Thinking...", bluecolor);
	textures[REDWINS]	= TTF_RenderText_Solid(font, "Red wins!", redcolor);
	textures[BLACKWINS]	= TTF_RenderText_Solid(font, "Black wins!", blackcolor);
	textures[EXIT]		= TTF_RenderText_Solid(font, "Exit", blackcolor);
	TTF_CloseFont(font);


	Uint32 rgb;

	rgb = SDL_MapRGB(textures[SELECTION]->format, 0, 0, 150);
	SDL_FillRect(textures[SELECTION], NULL, rgb);

	rgb = SDL_MapRGB(textures[BOX]->format, 0, 150, 0);
	SDL_FillRect(textures[BOX], NULL, rgb);

	rgb = SDL_MapRGB(textures[BADBOX]->format, 150, 0, 0);
	SDL_FillRect(textures[BADBOX], NULL, rgb);

	rgb = SDL_MapRGB(textures[SIDEBAR]->format, 200, 200, 200);
	SDL_FillRect(textures[SIDEBAR], NULL, rgb);

	for (map<char, SDL_Surface*>::iterator i = textures.begin();i != textures.end();++i) {
		if (i->second == NULL)
			return false;
	}

	other[BOARD].x = other[BOARD].y = 0;
	other[BOARD].w = (Uint16)textures[BOARD]->w;
	other[BOARD].h = (Uint16)textures[BOARD]->h;

	other[BOX].x = other[BADBOX].x = other[SELECTION].x = -1;
	other[BOX].y = other[BADBOX].y = other[SELECTION].y = -1;
	other[BOX].w = other[BADBOX].w = other[SELECTION].w = SPACE_W;
	other[BOX].h = other[BADBOX].h = other[SELECTION].h = SPACE_H;

	other[SIDEBAR].w = SCREEN_W - other[BOARD].w;
	other[SIDEBAR].h = SCREEN_H;
	other[SIDEBAR].x = other[BOARD].w;
	other[SIDEBAR].y = 0;

	ui[REDTURN].x = other[SIDEBAR].x + other[SIDEBAR].w/2 - Sint16(textures[REDTURN]->w/2);
	ui[REDTURN].y = 20;
	ui[REDTURN].w = (Uint16)textures[REDTURN]->w;
	ui[REDTURN].h = (Uint16)textures[REDTURN]->h;

	ui[BLACKTURN].x = other[SIDEBAR].x + other[SIDEBAR].w/2 - Sint16(textures[BLACKTURN]->w/2);
	ui[BLACKTURN].y = 20;
	ui[BLACKTURN].w = (Uint16)textures[BLACKTURN]->w;
	ui[BLACKTURN].h = (Uint16)textures[BLACKTURN]->h;

	ui[THINKING].x = other[SIDEBAR].x + other[SIDEBAR].w/2 - Sint16(textures[THINKING]->w/2);
	ui[THINKING].y = ui[REDTURN].y + 20;
	ui[THINKING].w = (Uint16)textures[THINKING]->w;
	ui[THINKING].h = (Uint16)textures[THINKING]->h;

	ui[BLACKWINS].x = other[SIDEBAR].x + other[SIDEBAR].w/2 - Sint16(textures[BLACKWINS]->w/2);
	ui[BLACKWINS].y = ui[BLACKTURN].y;
	ui[BLACKWINS].w = (Uint16)textures[BLACKWINS]->w;
	ui[BLACKWINS].h = (Uint16)textures[BLACKWINS]->h;

	ui[REDWINS].x = other[SIDEBAR].x + other[SIDEBAR].w/2 - Sint16(textures[REDWINS]->w/2);
	ui[REDWINS].y = ui[REDTURN].y;
	ui[REDWINS].w = (Uint16)textures[REDWINS]->w;
	ui[REDWINS].h = (Uint16)textures[REDWINS]->h;

	buttons[EXIT].w = (Uint16)textures[EXIT]->w;
	buttons[EXIT].h = (Uint16)textures[EXIT]->h;
	buttons[EXIT].x = other[SIDEBAR].x + other[SIDEBAR].w/2 - Sint16(textures[EXIT]->w/2);
	buttons[EXIT].y = SCREEN_H - buttons[EXIT].h - 10;

	//Init p1's checkers
	Uint16 w = (Uint16)textures[RED]->w;
	Uint16 h = (Uint16)textures[RED]->h;
	for (char i = 'a'; i != 'M'; ++i) {
		UpdateChecker(i);
		checkers[i].w = w;
		checkers[i].h = h;
		if (i == 'l') {	//Init p2's checkers
			i = 'A'-1;
			w = (Uint16)textures[BLACK]->w;
			h = (Uint16)textures[BLACK]->h;
		}
	}
	return true;
}

void Renderer::UpdateMouse(Sint16 px, Sint16 py) {
	if (px < other[SIDEBAR].x) {
		onBoard = true;
		other[BOX].x = other[BADBOX].x = Snap(px);
		other[BOX].y = other[BADBOX].y = Snap(py);
	} else {
		onBoard = false;
		other[BADBOX].x = other[BADBOX].y = -1;
		other[BOX].x = px;
		other[BOX].y = py;
	}
}

void Renderer::UpdateSelect(int px, int py) {
	if (px == -1) {
		other[SELECTION].x = other[SELECTION].y = -1;
		return;
	}
	other[SELECTION].x = Snap(ToScreenCoord(px));
	other[SELECTION].y = Snap(ToScreenCoord(py));
}

void Renderer::UpdateChecker(char key) {
	const Board& board = System::getInstance().board;
	const Checker* checker = board.getChecker(key);

	if (!checker) {
		checkers[key].x = checkers[key].y = -1;
		return;
	}

	checkers[key].x = ToScreenCoord(checker->getX());
	checkers[key].y = ToScreenCoord(checker->getY());
}

void Renderer::UpdateCheckers() {
	for (map<char, SDL_Rect>::iterator i = checkers.begin(); i != checkers.end();++i)
		UpdateChecker(i->first);
}

bool Renderer::isExit(int x, int y) {
	return Inside(x, y, buttons[EXIT].x, buttons[EXIT].w, buttons[EXIT].y, buttons[EXIT].h);
}

Renderer::~Renderer() {
	for (map<char, SDL_Surface*>::iterator i = textures.begin(); i != textures.end(); ++i) {
		SDL_FreeSurface(i->second);
		i->second = NULL;
	}
	//Display is freed by SDL_Quit and must not be freed by anything else
}