
#include "state.h"
#include "board.h"
#include "move.h"
#include "checker.h"
#include "enums.h"
#include "system.h"
#include "player.h"

#include <vector>
#include <fstream>
using namespace std;

long int AlphaBeta::Analyze(State* analyzeMe, int max, int min, const Player* curPlayer, int curDepth) {

	long int currentHeuristic = analyzeMe->getHeuristic();
	State* child = NULL;

	if (currentHeuristic == MAX_WIN || currentHeuristic == MIN_WIN) {
		if (bestDepth == START_DEPTH || curDepth < bestDepth)
			bestDepth = curDepth;
		return currentHeuristic;
	}

	if (curDepth >= curMaxDepth)
		return currentHeuristic;

	char curChecker;
	char lastChecker;
	const Player* opponent = System::getInstance().getOpponent(curPlayer);
	Direction dir;
	int nextDepth = curDepth + 1;

	if (curPlayer->getType() == MIN_PLAYER) {
		curChecker = 'a';
		lastChecker = 'l';	//one past end of checkers
		dir = DIR_DN_LEFT;
	} else {
		curChecker = 'A';
		lastChecker = 'L';	//one past end of checkers
		dir = DIR_UP_LEFT;
	}
	lastChecker++;
	bool success;

	for ( ; curChecker != lastChecker; curChecker++ ) {
		success = false;
		for (int i = 0;i < 2;i++) {
			//It's my turn now, get my moves
			child = analyzeMe->GenerateChild(curPlayer, curChecker, dir, false, false);

			//This child doesn't belong to this curPlayer or is invalid, go to the next one
			if (!child) {
				dir = SwapLeftRight(dir);
				continue;
			}

			//See what my opponent can do if I make said move
			currentHeuristic = Analyze(child, max, min, opponent, nextDepth);

			if (curPlayer->getType() == MIN_PLAYER) {	//it is min's turn
				if (currentHeuristic < min) {
					min = currentHeuristic;	//Update the min curPlayer's best move
					analyzeMe->heuristic = min;
					success = true;
				}

				if (max >= min) {		//Cutoff (this move sucks)
					delete child;
					return min;
				}

			} else {	//it is max's turn
				if (currentHeuristic > max) {
					max = currentHeuristic;
					analyzeMe->heuristic = max;
					success = true;
				}

				if (max >= min)	{ //Cutoff
					delete child;
					return max;
				}
			}
			delete child;
			child = NULL;
			dir = SwapLeftRight(dir);
		}
	}

	//Return best moves
	if (curPlayer->getType() == MIN_PLAYER)
		return min;

	return max;
}

//FIXME this is paste :P
State* AlphaBeta::getBestFor(Player* curPlayer, char index, Board* rootBoard, bool startup) {

	if (startup)
		Startup(curPlayer, rootBoard);

	const Player* opponent = System::getInstance().getOpponent(curPlayer);
	State* child;
	Direction dir;
	long int curValue;

	if (curPlayer->getType() == MIN_PLAYER)
		dir = DIR_DN_LEFT;
	else
		dir = DIR_UP_LEFT;

	bool success;
	long min = MAX_WIN;
	long max = MIN_WIN;

	for (int i = 0;i < 2;i++) {
		success = false;
		//Generate a move of mine
		child = root->GenerateChild(curPlayer, index, dir, true, false);

		//This child doesn't belong to this player or move was invalid
		if (!child) {
			dir = SwapLeftRight(dir);
			continue;
		}

		//Now see what my opponent can do for this particular move.
		oldBestDepth = bestDepth;
		curValue = Analyze(child, MIN_WIN, MAX_WIN, opponent, 1);
		if (curPlayer->getType() == MIN_PLAYER) {
			if (curValue < min) {
				min = curValue;
				success = true;
			} else if (curValue == MIN_WIN) {	//shortest path to win, please!
				if (bestDepth < oldBestDepth)
					success = true;
			}
		} else { //MAX_PLAYER
			if (curValue > max) {
				max = curValue;
				success = true;
			} else if (curValue == MAX_WIN) {	//shortest path to win
				if (bestDepth < oldBestDepth)
					success = true;
			}
		}

		if (success || !best) {
			delete best;
			best = child;
		} else {
			delete child;
		}
		dir = SwapLeftRight(dir);
	}

	return best;
}

void AlphaBeta::Startup(Player* p, Board* rootBoard) {

	if (p->getType() == MIN_PLAYER)
		curMaxDepth = maxDepthMin;
	else
		curMaxDepth = maxDepthMax;

	if (!root) {
		if (p->getType() == MIN_PLAYER)
			root = new State(NULL, rootBoard, 'z', DIR_NODIR, MAX_WIN);
		else if (p->getType() == MAX_PLAYER)
			root = new State(NULL, rootBoard, 'z', DIR_NODIR, MIN_WIN);
		root->setPlayer(System::getInstance().getOpponent(p));
	}
}

State* AlphaBeta::getBestStart(Player* curPlayer, Board* rootBoard) {

	long int curValue;
	char curChecker;
	char lastChecker;
	Direction dir;
	State* child;
	const Player* opponent = System::getInstance().getOpponent(curPlayer);

	Startup(curPlayer, rootBoard);

	if (curPlayer->getType() == MIN_PLAYER) {
		curChecker = 'a';
		lastChecker = 'm';	//one past end of checkers
		dir = DIR_DN_LEFT;
	} else {
		curChecker = 'A';
		lastChecker = 'M';	//one past end
		dir = DIR_UP_LEFT;
	}

	long int min = MAX_WIN;
	long int max = MIN_WIN;
	bool success;

	for ( ; curChecker != lastChecker; curChecker++) {
		for (int i = 0;i < 2;i++) {
			success = false;
			//Generate a move of mine
			child = root->GenerateChild(curPlayer, curChecker, dir, false, false);

			//This child doesn't belong to this player or move was invalid
			if (!child) {
				dir = SwapLeftRight(dir);
				continue;
			}

			//Now see what my opponent can do for this particular move.
			oldBestDepth = bestDepth;
			curValue = Analyze(child, MIN_WIN, MAX_WIN, opponent, 1);
			if (curPlayer->getType() == MIN_PLAYER) {
				if (curValue < min) {
					min = curValue;
					success = true;
				} else if (curValue == MIN_WIN) {	//shortest path to win, please!
					if (bestDepth < oldBestDepth)
						success = true;
				}
			} else { //MAX_PLAYER
				if (curValue > max) {
					max = curValue;
					success = true;
				} else if (curValue == MAX_WIN) {	//shortest path to win
					if (bestDepth < oldBestDepth)
						success = true;
				}
			}

			if (success || !best) {
				delete best;
				best = child;
			} else {
				delete child;
			}
			dir = SwapLeftRight(dir);
		}
	}
	//root->heuristic = best->heuristic;
	return best;
}

void AlphaBeta::Cleanup() {
	delete root;
	root = NULL;
	delete best;
	best = NULL;
	bestDepth = START_DEPTH;
	oldBestDepth = START_DEPTH;
}

AlphaBeta::AlphaBeta() : root(NULL), best(NULL), bestDepth(START_DEPTH), oldBestDepth(START_DEPTH) {}

void AlphaBeta::Init(int pmaxDepthMin, int pmaxDepthMax) {
	maxDepthMin = pmaxDepthMin;
	maxDepthMax = pmaxDepthMax;
}

AlphaBeta::~AlphaBeta() {
	Cleanup();
}

void AlphaBeta::WriteBest() {
/*	ofstream file("rating.txt");
	file.close();
	State* cur = bestbtm;
	while (cur != NULL) {
		file.open("rating.txt", ios::app);
		file << " depth: " << cur->getDepth() << endl
			 << "heuris: " << cur->getHeuristic() << endl
			 << "player: " << cur->player->getType() << endl
			 << "  move: " << cur->checker << " to " << cur->dir << endl;
		file.close();
		cur->myBoard->Draw(true);
		cur = cur->getParent();
	}*/
}

vector<State*>::size_type State::getNumChildren() {
	//return children.size();
	return 0;
}

State* State::getParent() {
	return parent;
}

State* State::getChild() {
	//return children.at(0);
	return this;
}

long int State::getHeuristic() {
	return heuristic;
}

int State::getDepth() {
	return depth;
}

void State::GenerateChildren(const Player* justMoved) {
	char curChecker;
	char lastChecker;
	Direction dir;

	if (justMoved->getType() == MIN_PLAYER) {
		curChecker = 'a';
		lastChecker = 'l';
		dir = DIR_DN_LEFT;
	} else {
		curChecker = 'A';
		lastChecker = 'L';
		dir = DIR_UP_LEFT;
	}
	lastChecker++;

	for (;curChecker != lastChecker; ++curChecker) {
		for (int i = 0;i < 2; ++i) {
			GenerateChild(justMoved, curChecker, dir);
			dir = SwapLeftRight(dir);
		}
	}
}

State* State::GenerateChild(const Player* needsToMove, char checkerIndex, Direction dir, bool jumpOnly, bool addToChildren) {
	Move move;
	Checker* originalChecker;
	Checker* newChecker;
	State* state;
	Board* board;
	MoveCode doCode;

	originalChecker = myBoard->getChecker(checkerIndex);
	if (originalChecker == NULL)	//No such checker
		return NULL;

	if (originalChecker->getOwner() != needsToMove) //Don't generate states for person whose turn it's not
		return NULL;

	//Create a simulation of the board, make a copy of checker
	board = new Board(myBoard, originalChecker);
	newChecker = board->getChecker(checkerIndex);	//get our simulated checker

	move.Init(newChecker->getX(), newChecker->getY(), dir, newChecker, board);
	doCode = move.Do();
	if (!move.isBad(doCode) &&
		( (!jumpOnly) || (jumpOnly && move.isJump(doCode)) ) )
	{
		state = new State(this, board, newChecker->getIndex(), dir, board->Rate(needsToMove, doCode, /*newChecker,*/ dir));
		state->setPlayer(needsToMove);
		//FIXME this is kind of a hack
		//generate multijump followthroughs and add them as children for the new state
		//the move was a multijump and we just started it
		if (doCode == MC_MULTIJUMP && !jumpOnly) {
			this->setMultijump();

			State* child = state;
			state = NULL;
			State* gchild = child;
			bool lastdirwassuccess;
			Direction dir2 = dir;

			//follow the multijump to the end, so we can compare the result!
			while (child) {
				lastdirwassuccess = false;
				for (int i = 0; i < 2; ++i) {
					child = gchild->GenerateChild(needsToMove, checkerIndex, dir2, true);
					if (child) {	//this child was a valid follow-up for our multijump

						//but it sucks worse than the other dir
						if (lastdirwassuccess) {	//this is here b/c we have to follow multijumps to end
													//so we can only compare children of the right depth, but im not sure what depth
													//is in this mess tbh
							if ( (player->getType() == MAX_PLAYER && child->getHeuristic() < gchild->getHeuristic())
								||	(player->getType() == MIN_PLAYER && child->getHeuristic() > gchild->getHeuristic()) )
							{
								delete child;
								child = gchild;	//reset!
								break;
							}
						}
						lastdirwassuccess = true;

						delete gchild;	//delete old good child
						gchild = child;
						delete state;
						state = NULL;
					}
				}
				dir2 = SwapLeftRight(dir2);
			}
			//one of the children has to be good... right?
			gchild->depth = this->depth+1;
			gchild->parent = this;
			gchild->dir = dir;	//the initial dir we move to start our multijump
//			if (addToChildren)
//				children.push_back(gchild);
			delete state;
			return gchild;
		}

//		if (addToChildren)
//			children.push_back(state);
		return state;
	}

	delete board;	//Delete the board (this should also delete its simulated checker)
	return NULL;
}

State::State(State* pparent, long int pheuristic)
	: parent(pparent), myBoard(NULL), checker('z'), dir(DIR_NODIR), heuristic(pheuristic), depth(0), multijump(false)
{}

State::State(State* pparent, Board* pmyBoard, char pchecker, Direction pdir, long int pheuristic)
	: parent(pparent), myBoard(pmyBoard), checker(pchecker), dir(pdir), heuristic(pheuristic)
{
	if (parent)
		depth = parent->depth + 1;
	else
		depth = 0;

	if (!myBoard)
		heuristic = 0;	//This should never happen
}

State::~State() {
//	children.clear();
	if (myBoard && myBoard->isSimulation()) {
		delete myBoard;
		myBoard = NULL;
	}
}
