
#ifndef STATE_H
#define STATE_H

#include <vector>
#include "enums.h"
#include "player.h"
using namespace std;

class Board;
class State;

class AlphaBeta {
	int maxDepthMin;
	int maxDepthMax;
	int curMaxDepth;
	State* root;
	State* best;
	int bestDepth;
	int oldBestDepth;
	static const int START_DEPTH = 99999;
	//State* bestbtm;
	//State* oldbestbtm;

public:
	void Cleanup();
	void Startup(Player* p, Board* rootBoard);
	State* getBest() { return best; }

	//curPlayer is the player who needs to take a turn
	long int Analyze(State* analyzeMe, int max, int min, const Player* curPlayer, int curDepth);
	State* getBestStart(Player* curPlayer, Board* rootBoard);
	State* getBestFor(Player* curPlayer, char index, Board* rootBoard, bool startup = false);
	AlphaBeta();
	void Init(int pmaxDepthMin, int pmaxDepthMax);
	~AlphaBeta();
	void WriteBest();
};

class State {
protected:
	State* parent;
	Board* myBoard;
	const Player* player;
//	vector<State*> children;
	long int heuristic;
	int depth;
	char checker;	//Checker we're moving
	Direction dir;	//Dir it moves to get here from the last state
	bool multijump;

	void GenerateChildren(const Player* justMoved);
	State* GenerateChild(const Player* needsToMove, char checkerIndex, Direction dir, bool jumpOnly = false, bool addToChildren = true);

	friend class AlphaBeta;

public:
	State* getParent();
	long int getHeuristic();
	int getDepth();
	char getChecker() const { return checker; }
	Direction getDir() const { return dir; }
	vector<State*>::size_type getNumChildren();
	State* getChild();
	void setPlayer(const Player* pplayer) { player = pplayer; }
	const Player* getPlayer() { return player; }
	void setMultijump(bool pto = true) { multijump = pto; }
	bool getMultijump() { return multijump; }
	State(State* pparent, long int pheuristic);
	State(State* pparent, Board* pmyBoard, char pchecker, Direction pdir, long int pheuristic);
	~State();
};

#endif