
#include "system.h"
#include "sdl.h"
#include "enums.h"
#include "move.h"
#include "state.h"
#include "player.h"

//#include <fstream>
#include <iostream>
using namespace std;

void System::OutputMessage() {
	if (message == "")
		return;
	cout << message << endl;
	message = "";
}

void System::Draw() {
	if (isQuitTime())
		return;
//	system("cls");
	OutputMessage();
//	board.Draw();
	renderer.Draw();
}

SystemReturn System::DoTurn() {

	totalTurns++;		
	State* bestStart = NULL;
	Direction dirDecision = DIR_NODIR;
	renderer.UpdateSelect(-1, -1);

	if (turn->isComputer()) {
		turn->toggleThinking();
		renderer.Draw();
		bestStart = alphaBeta.getBestStart(turn, &board);
		if (!bestStart) {
			cout << "No valid move..." << endl;
			message = "Stalemate...";
			return SYS_WIN;
		}
		dirDecision = bestStart->getDir();
		turn->toggleThinking();
		renderer.Draw();
		if (getOpponent(turn)->isComputer()) {
			while (selectx != -2) 
				CheckEvents(true);
		}
		selectx = -1;
//		SDL_Delay(250);
	}

	if (turn->getType() == MIN_PLAYER)
		cout << "Player 1 (top), ";
	else
		cout << "Player 2 (bottom), ";

	cout << "select a checker: ";
	char input = 'z';

	Checker* selection;

	if (turn->isComputer()) {
//		if (!CheckEvents(false, false)) {
//			Quit();
//			return SYS_QUIT;
//		}
		input = bestStart->getChecker();
		selection = board.getChecker(input);
	} else {
		//cin >> input;
		if (!SelectPrompt())
			return SYS_QUIT;
		selection = board.getChecker(selectx, selecty);
	}

	if (input == 'q') {
		Quit();
		return SYS_QUIT;
	}

	if (!selection) {
		message = "-\nInvalid selection.";
		return SYS_BAD;
	}

	cout << selection->getIndex() << endl;
	if (selection->getOwner() != turn) {
		message = "That's not yours!";
		return SYS_BAD;
	}

	renderer.UpdateSelect(selection->getX(), selection->getY());
	SystemReturn retval = DoTurnMove(selection, dirDecision);

	return retval;
}

SystemReturn System::DoTurnMove(Checker* selection, Direction dir, bool forceJump) {

	static Checker* lastSelect = NULL;
	Move move;
	MoveCode mc;

	if (turn->isComputer()) {
		if (selection == NULL) {
//			alphaBeta.Cleanup();
//			alphaBeta.SetRoot(new State(NULL, &board, 3, 2, 'z', DIR_NODIR, 0));

			//need a 'get nearest jumpable' thing
			dir = alphaBeta.getBestFor(turn, lastSelect->getIndex(), &board, true)->getDir();
		}
		//dir = alphaBeta.getBest(turn)->getDir();
	}

	if (selection == NULL)
		selection = lastSelect;
	else
		lastSelect = selection;

	dir = MoveCheckerPrompt(dir, selection);

	if (dir == DIR_ABORT)
		return SYS_BAD;

	if (isQuitTime())
		return SYS_QUIT;

	move.Init(selection->getX(), selection->getY(), dir, selection, &board);

	if (forceJump) {
		mc = move.CalcType();
		if (!move.isJump(mc)) {
			message = "Must continue the jump!";
			return SYS_BAD;
		}
	}

	mc = move.Do();
	renderer.UpdateSelect(selection->getX(), selection->getY());
	switch (mc) {
		case MC_KING:
		case MC_JUMPDONEKING:
			if (turn->getType() == MIN_PLAYER) {
				message = "\n====================\nTop player wins!\n====================\n";
				win = 1;
			} else {
				message = "\n=====================\nBottom player wins!\n=====================\n";
				win = 2;
			}
			if (mc == MC_JUMPDONEKING)
				message += "(By jump)";
			else
				message += "(By walk-in)";
			renderer.UpdateCheckers();
			alphaBeta.Cleanup();
			return SYS_WIN;
		case MC_JUMPABLE:
			//? Is this _supposed_ to be a fall-through to BADMOVE_WALL?
			//Can we even _get_ this value?
		case MC_BADMOVE_WALL:
			message = "Cannot move there.";
			alphaBeta.Cleanup();
			return SYS_BAD;
		case MC_BADMOVE_NME:
			message = "Can't jump that checker.";
			alphaBeta.Cleanup();
			return SYS_BAD;
		case MC_MULTIJUMP:
			message = "Multi-jump possible!";
			renderer.UpdateCheckers();
			return SYS_CONTINUE;
		case MC_JUMPDONE:
			message = "Jumped a checker!";
			renderer.UpdateCheckers();
		case MC_MOVEOK:	//fall through.
			if (mc == MC_MOVEOK)
				renderer.UpdateChecker(selection->getIndex());
			alphaBeta.Cleanup();
			return SYS_GOOD;
	}
	return SYS_BAD;
}

Direction System::MoveCheckerPrompt(Direction dir, Checker* selection) {
	char input;

	for (;;) {
		cout << "Move (l)eft, (r)ight or (a)bort? ";
		if (dir != DIR_NODIR) {	//Computer
			if (dir == DIR_UP_LEFT || dir == DIR_DN_LEFT)
				cout << "L" << endl;
			else
				cout << "R" << endl;
//			if (!CheckEvents(false, false))
//				return DIR_NODIR;
			return dir;
		}

		if (!SelectPrompt())
			return DIR_NODIR;

		if (selectx < selection->getX()) {
			input = 'l';
		} else if (selectx > selection->getX()) {
			input = 'r';
		} else {
			input = 'a';
		}

		if (selection->getOwner() != turn)
			input = 'a';
		else if ( (selection->getOwner()->getType() == MAX_PLAYER) && (selecty >= selection->getY()) )
			input = 'a';
		else if ( (selection->getOwner()->getType() == MIN_PLAYER) && (selecty <= selection->getY()) )
			input = 'a';

		//cin >> input;
		//cin.ignore(100, '\n');

		cout << input << endl;
		if (input == 'l') {
			if (turn->getType() == MIN_PLAYER)
				return DIR_DN_LEFT;
			return DIR_UP_LEFT;
		} else if (input == 'r') {
			if (turn->getType() == MIN_PLAYER)
				return DIR_DN_RIGHT;
			return DIR_UP_RIGHT;
		} else if (input == 'a') {
			cout << "Aborting selection." << endl;
			return DIR_ABORT;
		} else if (input == 'q') {
			cout << "Quitting..." << endl;
			return DIR_NODIR;
		} else {
			cout << "Invalid direction." << endl;
		}
	}
	//return DIR_ABORT;
}

void System::ChangeTurn() {
	cout << endl;
	if (turn == &players[0])
		turn = &players[1];
	else
		turn = &players[0];
}

const Player* System::getPlayer(PlayerType which) const {
	return (which == MIN_PLAYER) ? &players[0] : &players[1];
}

const Player* System::getOpponent(const Player* which) const {
	return getPlayer(!(which->getType()));
}

System::System() : turn(&players[0]), quit(false), win(0) {}

void System::Init(int maxDepthMinplayer, int maxDepthMaxplayer, bool minplayerComp, bool maxplayerComp) {
	players[0].Init(minplayerComp, MIN_PLAYER);
	players[1].Init(maxplayerComp, MAX_PLAYER);
	alphaBeta.Init(maxDepthMinplayer, maxDepthMaxplayer);
	renderer.Init();
	totalTurns = 0;
}

//Returns false on exit
bool System::CheckEvents(bool selectMode, bool block) {

	if (block) {
		SDL_WaitEvent(&sdlEvent);
		if (SwitchEvents(selectMode) == false)
			return false;
		renderer.Draw();
	} else {
		while (SDL_PollEvent(&sdlEvent)) {
			if (SwitchEvents(selectMode) == false)
				return false;
			renderer.Draw();
		}
	}
	return true;
}

bool System::SwitchEvents(bool selectMode) {
	switch (sdlEvent.type) {
		case SDL_QUIT:
			Quit();
			return false;
		case SDL_KEYDOWN:
			switch (sdlEvent.key.keysym.sym) {
				case SDLK_ESCAPE:
					Quit();
					return false;
			};
		case SDL_MOUSEBUTTONUP:

			if (sdlEvent.button.button == SDL_BUTTON_LEFT) {
				if (renderer.isExit(sdlEvent.motion.x, sdlEvent.motion.y)) {
					selectx = -2;
					Quit();
					return false;
				}

				if (!selectMode)
					return true;

				if (getTurn()->isComputer() == false) {
					selectx = int(sdlEvent.motion.x/60);
					selecty = int(sdlEvent.motion.y/60);
				} else {
					selectx = -2;
				}
			} else if (sdlEvent.button.button == SDL_BUTTON_RIGHT) {
				alphaBeta.WriteBest();
			}
			return true;
		case SDL_MOUSEMOTION:
			renderer.UpdateMouse(sdlEvent.motion.x, sdlEvent.motion.y);
			return true;
		default:
			return true;
	};
}

//Returns true on success
bool System::SelectPrompt() {
	selectx = selecty = -1;
	while (selectx == -1) {
		renderer.Draw();
		if (!CheckEvents(true))
			return false;
	}
	return true;
}

void System::Quit() {
	quit = true;
}

inline bool System::isQuitTime() {
	return quit;
}


/*bool System::Init(char* imageFile) {
	return true;
}*/

/*bool System::AddImage(char* filename) {
	return true;
}*/

/*void System::Draw() {
	SDL_Rect rc;
	rc.x = rc.y = 0;
	rc.w = imageSurfaces[0]->w;
	rc.h = imageSurfaces[0]->h;

	SDL_BlitSurface(imageSurface, &rc, displaySurface, &rc);

	//Update entire screen
	SDL_UpdateRect(displaySurface, 0, 0, 0, 0);
}

bool System::Init(char* imageFile) {
	ofstream file(imageFile);
	
	if (!file) {
		fprintf(stderr, "Could not open file '%s' for system init.\n");
		return false;
	}

	if (!AddImage("images/default.png")) {
		fprintf(stderr, "Could not load default image: 'images/default.png'.\n");
		return false;
	}

	char buffer[MAX_STR];

	file.getline(buffer, MAX_STR);
	while (file) {
		if (!AddImage(buffer))
			fprintf(stderr, "Error loading image '%s'.\n", buffer);
		file.getline(buffer, MAX_STR);
	}
	return true;
}

bool System::AddImage(char* filename) {
	SDL_Surface* loaded = IMG_Load(filename);
	if (!loaded)
		return false;
	imageSurfaces.push_back(loaded);
	return true;
}

int System::ProcessInput() {
	switch (sdlEvent.type) {
		case SDL_QUIT:
			return -1;
			break;
	}
	return 0;
}*/