
#ifndef SYSTEM_H
#define SYSTEM_H

//#include "sdl.h"

#include "checker.h"
#include "board.h"
#include "state.h"
#include "player.h"
#include "renderer.h"
#include <string>

using namespace std;

extern enum SystemReturn;
const size_t MAX_STR = 1028;

class System {
private:
	Board board;
	AlphaBeta alphaBeta;
	Renderer renderer;
	SDL_Event sdlEvent;

	string message;
	Player* turn;
	void OutputMessage();
	Player players[2];
	int selectx, selecty;
	bool quit;
	int totalTurns;
	int win;

	friend class Renderer;

	inline const Board& getBoard() const;
	inline const Player* getTurn() const;

public:

	static System& getInstance() {
		static System sys;
		return sys;
	}

	System();
	void Init(int maxDepthMinplayer, int maxDepthMaxplayer, bool minplayerComp, bool maxplayerComp);

	void Draw();
	SystemReturn DoTurn();
	SystemReturn DoTurnMove(Checker* selection, Direction dir = DIR_NODIR, bool forceJump = false);
	Direction MoveCheckerPrompt(Direction dir = DIR_NODIR, Checker* selection = NULL);
	void ChangeTurn();
	const Player* getPlayer(PlayerType which) const;
	const Player* getOpponent(const Player* which) const;
	bool CheckEvents(bool selectMode = false, bool block = true);
	bool SwitchEvents(bool selectMode);
	bool SelectPrompt();
	void Quit();
	inline bool isQuitTime();
};

#include "system.inl"

#endif